﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using Telerik.Windows.Controls.Map;
using Telerik.Windows.Data;

namespace DisposeMapControl
{
    /// <summary>
    /// Interaction logic for MapControl.xaml
    /// </summary>
    public partial class MapControl : UserControl, IDisposable
    {
        private readonly Random _random = new Random();
        private readonly RadObservableCollection<object> _mapEntities;
        private Timer _updater;

        public MapControl()
        {
            InitializeComponent();
            DataContext = this;

            _mapEntities = new RadObservableCollection<object>();
            _mapEntities.AddRange(CreateMapEntities());

            RadMap.Items.Add(CreateVisualizationLayer(_mapEntities, "LineItemTemplate"));

            _updater = new Timer(o => UpdateMap(null, null));
            _updater.Change(10, 500);
        }

        private void UpdateMap(object sender, EventArgs e)
        {
            Dispatcher.BeginInvoke(new Action(UpdateMapEntities));
        }

        public void UpdateMapEntities()
        {
            _mapEntities.SuspendNotifications();
            _mapEntities.Clear();
            _mapEntities.AddRange(CreateMapEntities());
            _mapEntities.ResumeNotifications();
        }

        public IEnumerable<object> CreateMapEntities()
        {
            return Enumerable.Range(0, 20).Select(x =>
            {
                var mapPoints = new LocationCollection();
                for (var i = 0; i < 3; i++)
                {
                    mapPoints.Add(GetRandomLocation());
                }
                return new MapItem {Location = GetRandomLocation(), MapPoints = mapPoints};
            });
        }

        public Location GetRandomLocation()
        {
            var lat = _random.Next(-10, 10);
            var lon = _random.Next(-10, 10);
            return new Location(lat, lon);
        }

        public void Dispose()
        {
            RadMap.Dispose();
        }

        private VisualizationLayer CreateVisualizationLayer(RadObservableCollection<object> mapEntities, string dataTemplate)
        {
            var layer = new VisualizationLayer()
            {
                ItemTemplate = (DataTemplate)Resources[dataTemplate]
            };

            layer.Loaded += (o, args) =>
            {
                layer.ItemsSource = mapEntities;
            };
            return layer;
        }
    }
}
